@@ -45,6 +45,7 @@ Email   forster@mathematik.uni-muenchen.de
 ** 2003-02-15:  Fdecode made safe wrt garbage collection
 ** 2003-11-21:  removed bug in vecdiv
 ** 2007-02-23:  changed negatevec to make it robust wrt gc
+** 2007-11-15:  function roundhalf: case of number exactly = 1/2
 */
 
 #include "common.h"
@@ -1380,6 +1381,7 @@ truc symb;
     numdata x1, y;
     int type;
     int prec, s1;
+    int rhalf;
 
     type = chknum(symb,argStkPtr);
     if(type == aERROR)
@@ -1410,7 +1412,14 @@ truc symb;
     }
     floshiftint(&x1);
     if(symb == roundsym) {
-        if(roundhalf(&y) == 1) {
+        rhalf = roundhalf(&y);
+        if(rhalf == 2) {
+            if((x1.digits[0] & 1) && (x1.len)) 
+                rhalf = 1;
+            else
+                rhalf = 0;
+        }
+        if(rhalf == 1) {
             x1.len = incarr(x1.digits,x1.len,1);
             x1.sign = s1;
         }
@@ -1460,15 +1469,36 @@ numdata *npt1, *npt2;
 /*------------------------------------------------------------------*/
 /*
 ** Ergibt 1 oder 0, je nachdem die in *nptr gegebene Zahl absolut
-** groesser oder kleiner 1/2 ist
+** groesser oder kleiner 1/2 ist.
+** If the number equals exactly 1/2, the return value is 2
 */
 PRIVATE int roundhalf(nptr)
 numdata *nptr;
 {
     long nn;
+    word2 *xx;
+    int i, bb, len;
 
-    nn = bit_length(nptr->digits,nptr->len);
-    return(nn >= -nptr->expo ? 1 : 0);
+    len = nptr->len;
+    if (len == 0)
+        return 0;
+    xx = nptr->digits;
+    nn = bit_length(xx,len);
+    if (nn < -nptr->expo)
+        return 0;
+    if (nn == -nptr->expo) {
+        for (i=0; i < len-1; i++) {
+            if (xx[i])
+                return 1; 
+        }
+        bb = (nn - 1) % 16;
+        if (xx[len-1] != (1 << bb))
+            return 1;
+        else
+            return 2;
+    }
+    else
+        return 1;
 }
 /*------------------------------------------------------------------*/
 PRIVATE void floshiftint(nptr)
