@@ -2,7 +2,7 @@
 /* file arity.c
 
 ARIBAS interpreter for Arithmetic
-Copyright (C) 1996-2003 O.Forster
+Copyright (C) 1996-2007 O.Forster
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -34,20 +34,22 @@ Email   forster@mathematik.uni-muenchen.de
 ** and prime generation (next_prime)
 **
 ** date of last change
-** 1997-12-09:	created by splitting the file aritx.c
-** 1997-12-22:	qs_factorize, big prime var
-** 1998-05-31:	fixed bug in function mkquadpol
-** 1998-10-11:	function next_prime
-** 2000-04-26:	fixed bug in ATARI version due to ptr diffs > 32K
+** 1997-12-09:  created by splitting the file aritx.c
+** 1997-12-22:  qs_factorize, big prime var
+** 1998-05-31:  fixed bug in function mkquadpol
+** 1998-10-11:  function next_prime
+** 2000-04-26:  fixed bug in ATARI version due to ptr diffs > 32K
 ** 2003-05-30:  fixed bug in qs_factorize
-** 2004-08-20:  Removed #define ETEST
+** 2004-08-20:  removed #define ETEST
+** 2007-08-20:  fixed bug in ecfactbpv
 */
 
 #include "common.h"
 
 PUBLIC void iniarity    _((void));
 PUBLIC void workmess   _((void));
-PUBLIC void tick   	_((int c));
+PUBLIC void tick    _((int c));
+PUBLIC int showvect  _((FILE *f, word2 *xx, int len));
 
 /*--------------------------------------------------------*/
 /********* fuer rho-Faktorisierung **********/
@@ -203,20 +205,7 @@ typedef struct {
     word2 *yy;
     int ylen;
 } EPOINT;
-/*-----------------------------------------------------------------*/
-
-#ifdef QTEST    /* only for testing */
-#define DBGFILE
-#endif
-#ifdef ETEST
-#define DBGFILE
-#endif
-#ifdef DBGFILE
-FILE *dbgf;
-#endif
-
 /*-------------------------------------------------------------------*/
-static char zeile[80];
 /*-------------------------------------------------------------------*/
 /* setbit and testbit suppose that vv is an array of word2 */
 #define setbit(vv,i)    vv[(i)>>4] |= (1 << ((i)&0xF))
@@ -233,7 +222,7 @@ PRIVATE truc Fcffact    _((int argn));
 PRIVATE int brillmorr   _((word2 *N, int len, unsigned v, word2 *fact));
 PRIVATE int bm_alloc    _((word2 *buf1, size_t len1, word2 *buf2, size_t len2,
                BMDATA *bmp, int alen, int qlen));
-PRIVATE int brill1  	_((word2 *N, int len, unsigned u, word2 *fact,
+PRIVATE int brill1      _((word2 *N, int len, unsigned u, word2 *fact,
                BMDATA *bmp, CFDATA *cfp, word2 *hilf));
 PRIVATE unsigned factorbase  _((word2 *N, int len, word2 *prim, int anz,
                 unsigned *pdivis));
@@ -253,16 +242,16 @@ PRIVATE int mpqsfactor  _((word2 *N, int len, word2 *fact));
 PRIVATE int qsfact1 _((word2 *N, int len, word2 *fact, BMDATA *bmp,
                SIEVEDATA *qsp, word2 *hilf));
 
-PRIVATE int ppexpo	_((unsigned B1, unsigned B2, word2 *xx));
+PRIVATE int ppexpo  _((unsigned B1, unsigned B2, word2 *xx));
 
-PRIVATE int startqq 	_((word2 *N, int len, unsigned srange,
+PRIVATE int startqq     _((word2 *N, int len, unsigned srange,
                word2 *qq, word2 *hilf));
 
-PRIVATE int nextqq  	_((word2 *N, int len, word2 *q0, int q0len,
+PRIVATE int nextqq      _((word2 *N, int len, word2 *q0, int q0len,
                 word2 *qq, word2 *hilf));
-PRIVATE int dosieve 	_((SIEVEDATA *qsp));
+PRIVATE int dosieve     _((SIEVEDATA *qsp));
 
-PRIVATE int p2sqrt  	_((word2 *p, int plen, word2 *x, int xlen,
+PRIVATE int p2sqrt      _((word2 *p, int plen, word2 *x, int xlen,
                 word2 *z, word2 *hilf));
 
 PRIVATE int mkquadpol   _((word2 *p, int plen, QPOLY *sptr, word2 *hilf));
@@ -292,7 +281,8 @@ PRIVATE int combinebp   _((word2 *N, int len, word4 prim, word2 *QQ, word2 *AA,
 PRIVATE void qs2mess    _((long n, int nf, int nf2));
 #endif
 
-PRIVATE truc Fecfactor	_((int argn));
+PRIVATE truc Fecfactor  _((int argn));
+PRIVATE truc _Fecfactor _((int argn));
 PRIVATE int ecfacta _((word2 *N, int len, word2 *aa, int alen,
                        word2 *xx, int xlen, unsigned *pbound, word2 *hilf));
 PRIVATE int ecbpvalloc   _((EPOINT *pEpoint, word2 *buf, size_t buflen,
@@ -331,6 +321,33 @@ PUBLIC void iniarity()
 #endif
 }
 /*-------------------------------------------------------------------*/
+#if 0
+#ifdef E2TEST    /* only for testing */
+FILE *elog;
+#endif
+#ifdef E2TEST
+elog = fopen("e2test.log","w");
+fprintf(elog,"N := "); showvect(elog,N,len);
+#endif
+#ifdef E2TEST
+fclose(elog);
+#endif
+#endif
+
+/*-------------------------------------------------------------------*/
+PUBLIC int showvect(logf,vect,len)
+FILE *logf;
+word2 *vect;
+int len;
+{
+    int i;
+    fprintf(logf,"0x");
+    for(i=len-1; i>=0; i--)
+        fprintf(logf,"%04X",vect[i]);
+    fprintf(logf,"\n");
+    return len;
+}
+/*-------------------------------------------------------------------*/
 #ifdef ETEST
 PRIVATE truc Feee()
 {
@@ -338,8 +355,8 @@ PRIVATE truc Feee()
     int alen, clen, xlen, sign, nlen;
     ECN ecN;
 
-	if(chkints(eeesym,argStkPtr-2,3) == aERROR)
-		return(brkerr());
+    if(chkints(eeesym,argStkPtr-2,3) == aERROR)
+        return(brkerr());
 
     nlen = bigref(argStkPtr-2,&N,&sign);
     alen = bigref(argStkPtr-1,&aa,&sign);
@@ -365,8 +382,8 @@ PRIVATE truc Fee1()
     ECN ecN;
     EPOINT PP;
 
-	if(chkints(eeesym,argStkPtr-2,3) == aERROR)
-		return(brkerr());
+    if(chkints(eeesym,argStkPtr-2,3) == aERROR)
+        return(brkerr());
 
     nlen = bigref(argStkPtr-2,&N,&sign);
     alen = bigref(argStkPtr-1,&aa,&sign);
@@ -408,11 +425,8 @@ PRIVATE truc Fee2()
     ECN ecN;
     EPOINT PP;
 
-#ifdef ETEST1
-dbgf = fopen("etest.log","w");
-#endif
-	if(chkints(eeesym,argStkPtr-3,4) == aERROR)
-		return(brkerr());
+    if(chkints(eeesym,argStkPtr-3,4) == aERROR)
+        return(brkerr());
 
     nlen = bigref(argStkPtr-3,&N,&sign);
     alen = bigref(argStkPtr-2,&aa,&sign);
@@ -444,10 +458,6 @@ dbgf = fopen("etest.log","w");
 
     len = ECNmult(&ecN,&PP,ex,exlen,hilf);
 
-#ifdef ETEST1
-fclose(dbgf);
-#endif
-
     if(len > 0)
         return mkint(0,AriBuf,len);
     else
@@ -1011,34 +1021,34 @@ CFDATA *cfp;
         cf0mess(fbase[baslen-1],baslen);
 
     easize = intsqrt(6*(word4)baslen);    /* ?! */
-	maxshrieks = (baslen < 100 ? 20 : 20 + (baslen/2 - 50)/bitlen(baslen));
+    maxshrieks = (baslen < 100 ? 20 : 20 + (baslen/2 - 50)/bitlen(baslen));
     for(v=1, count=count1=0; qrlen && count1<maxshrieks; v++) {
         if((v & 0xFF) == 0) {
-	        if(INTERRUPT) {
-	            setinterrupt(0);
-	            break;
-	        }
-	        if((v & 0x3FF) == 0) {
-	            if(doreport)
-	                counttick(v>>10,bmp);
-	        }
+            if(INTERRUPT) {
+                setinterrupt(0);
+                break;
+            }
+            if((v & 0x3FF) == 0) {
+                if(doreport)
+                    counttick(v>>10,bmp);
+            }
         }
         if(smoothea(cfp->QQ,fbase,baslen,easize)) {
-	        if((++count & 0x3) == 1)
-	            if(doreport)
-	                tick('.');
-	        res = bm_insert(bmp,cfp->QQ,cfp->qrsign,cfp->AA,hilf);
-	        if(res && gausselim(bmp)) {
-	            count1++;
-	            if(doreport)
-	                tick('!');
-	            k = getfactor(N,len,bmp,fact,hilf);
-	            if(k > 0) {
-	                if(doreport)
-	                    cf1mess(v,count);
-	                return(k);
-	            }
-	        }
+            if((++count & 0x3) == 1)
+                if(doreport)
+                    tick('.');
+            res = bm_insert(bmp,cfp->QQ,cfp->qrsign,cfp->AA,hilf);
+            if(res && gausselim(bmp)) {
+                count1++;
+                if(doreport)
+                    tick('!');
+                k = getfactor(N,len,bmp,fact,hilf);
+                if(k > 0) {
+                    if(doreport)
+                        cf1mess(v,count);
+                    return(k);
+                }
+            }
         }
         qrlen = cfracnext(N,len,cfp,hilf);
     }
@@ -1244,15 +1254,15 @@ int baslen;
     word2 r;
 
     qn = QQ[-1];
-	if(!qn)
-		return 0;
+    if(!qn)
+        return 0;
     cpyarr(QQ,qn,Q);
     for(i=0; i<baslen; i++) {
         p = *fbas++;
         while(modarr(Q,qn,p) == 0) {
             qn = divarr(Q,qn,p,&r);
-		}
-        if(qn == 1 && Q[0] == 1) 
+        }
+        if(qn == 1 && Q[0] == 1)
             return(1);
     }
     if(qn <= 2)
@@ -1278,9 +1288,9 @@ int qsign;
     word2 r;
     unsigned p;
     int qn, i, v, alen, baslen;
-	size_t bmrk;
+    size_t bmrk;
 
-	bmrk = bmp->rank;
+    bmrk = bmp->rank;
     Q = bmp->Qarr + bmrk*(bmp->qinc);
     qn = Q[-1] = QQ[-1];
     cpyarr(QQ,qn,Q);
@@ -1318,12 +1328,12 @@ int showvect(vect,len)
 word2 *vect;
 int len;
 {
-	int i;
+    int i;
     fprintf(dbgf,"0x");
     for(i=len-1; i>=0; i--)
-		fprintf(dbgf,"%04X",vect[i]);
-	fprintf(dbgf,"\n");
-	return len;
+        fprintf(dbgf,"%04X",vect[i]);
+    fprintf(dbgf,"\n");
+    return len;
 }
 #endif
 /*-------------------------------------------------------------------*/
@@ -1337,7 +1347,7 @@ BMDATA *bmp;
     unsigned pivot, minc;
     int i;
     int vn, v2n, baslen;
-	size_t rk;
+    size_t rk;
 
     minc = bmp->matinc;
     rk = bmp->rank;
@@ -1363,7 +1373,7 @@ BMDATA *bmp;
             break;
     }
     if(i == vn)
-		return(1);	/* then vect is identically zero */
+        return(1);  /* then vect is identically zero */
     /* find new pivot */
     for(i=rk; i<=baslen; i++) {
         pivot = piv[i];
@@ -1393,7 +1403,7 @@ BMDATA *bmp;
 {
     word2 *relat, *temp, *aux, *A, *AA, *Q, *QQ, *XX, *X1, *X2;
     int cmp, k;
-	size_t rk, ainc, qinc;
+    size_t rk, ainc, qinc;
     int count;
     int alen, qlen, q1len, tlen, xlen, x1len, x2len;
     int ll, lll;
@@ -1604,7 +1614,7 @@ int len;
     }
 #endif /* BIGPRIMES */
   nochmal:
-	srange = MINSRANGE;
+    srange = MINSRANGE;
     if(bitl > 40)
         srange += (bitl-40)*(3*bitl)/2;
     if(bitl > 80)
@@ -1614,7 +1624,7 @@ int len;
     }
 /*
     srange = bitl*100;
-	srange = MINSRANGE;
+    srange = MINSRANGE;
     if(bitl > 64)
         srange += (bitl-64)*(bitl/5)*16;
 */
@@ -1649,12 +1659,12 @@ int len;
 
     maxrows = bm_alloc(buf1,b1len,buf2,b2len,&bm,alen,qlen);
     b0 = maxrows/16;
-	b = 1 + (bitl*bitl)/384;
+    b = 1 + (bitl*bitl)/384;
     if(bitl > 160)
         b += (b*(bitl-160))/80;
     if(again)
         b -= 1;
-	if(b > b0)
+    if(b > b0)
         b = b0;
     bm.vlen = b;
     baslen = b*16 - 2;
@@ -1706,7 +1716,7 @@ SIEVEDATA *qsp;
     QSFBAS *fbp;
     word2 Q0[HLEN], Q1[HLEN], Work[2*ALEN+4];
     word2 *fbase, *fbroot, *QQ, *AA;
-	word2 *NN, *hilf1;
+    word2 *NN, *hilf1;
     sievitem *fblog, *sieve, *sptr;
     word4 cofac, v;
     int res, haveres, shrieks, maxshrieks;
@@ -1769,17 +1779,17 @@ SIEVEDATA *qsp;
         qsbigp->row = 0;
         cpyarr(Q0,qlen,qsbigp->Q0);
         qsbigp->Q0[-1] = qlen;
-    }   
+    }
 #endif
     target = bit_length(N,len)/2 + lbitlen((word4)srange) - tol;
 
     if(doreport)
         qs0mess(srange,fbase[baslen-1],baslen);
-	maxshrieks = (baslen < 64 ? 32 : 32 + (baslen/2 - 32)/bitlen(baslen));
+    maxshrieks = (baslen < 64 ? 32 : 32 + (baslen/2 - 32)/bitlen(baslen));
     for(v=1, count=count1=shrieks=0; shrieks<maxshrieks; v++) {
         if(INTERRUPT) {
-			setinterrupt(0);
-        	return -1;
+            setinterrupt(0);
+            return -1;
         }
         /* calculate polynomial for next sieving */
   nochmal:
@@ -1788,87 +1798,87 @@ SIEVEDATA *qsp;
         qlen = nextqq(N,len,Q1,qlen,Q0,hilf);
         res = mkquadpol(Q0,qlen,&Qpol,hilf);
         if(res > 0) {   /* Q0 divides N */
-			cpyarr(Q0,qlen,fact);
-			return(qlen);
+            cpyarr(Q0,qlen,fact);
+            return(qlen);
         }
         else if(res < 0) {  /* possibly Q0 not prime */
-			if(doreport)
-				tick('\'');
-			if(++shrieks>=maxshrieks)
-				break;
-			else
-				goto nochmal;
+            if(doreport)
+                tick('\'');
+            if(++shrieks>=maxshrieks)
+                break;
+            else
+                goto nochmal;
         }
         dosieve(qsp);
         /* collect sieve results */
         for(sptr=sieve,xi=-srange; xi<srange; xi++,sptr++) {
-	        if(*sptr >= target) {
-	            haveres = 0;
-	            Qpol.xi = xi;
-	            qvlen = quadvalue(&Qpol,QQ,&sgn);
-				if(!qvlen) {	/****** N square? ************/
-					if(doreport)
-						tick('`');
-					continue;
-				}
-	            cofac = smooth(QQ,fbase,baslen);
-	            if(cofac == 1) {
-	            	if((++count1 & 0x3) == 1)
-	                	if(doreport)
-	                		tick('.');
-	            	qresitem(&Qpol,AA);
-	            	haveres = 1;
-            	}
+            if(*sptr >= target) {
+                haveres = 0;
+                Qpol.xi = xi;
+                qvlen = quadvalue(&Qpol,QQ,&sgn);
+                if(!qvlen) {    /****** N square? ************/
+                    if(doreport)
+                        tick('`');
+                    continue;
+                }
+                cofac = smooth(QQ,fbase,baslen);
+                if(cofac == 1) {
+                    if((++count1 & 0x3) == 1)
+                        if(doreport)
+                            tick('.');
+                    qresitem(&Qpol,AA);
+                    haveres = 1;
+                }
 #ifdef BIGPRIMES
-            	else if(useBigprim && (cofac < bigpbound) && (cofac > 1)) {
-	            	res = hashbigp(qsbigp,cofac,&Qpol,&Qpol2,hilf);
-	            	if(res > 0) {
-	                	if((++count2 & 0x3) == 1)
-	                		if(doreport)
-	                    		tick(':');
-	                	qresitem(&Qpol,AA);
-	                	quadvalue(&Qpol2,QQ2,&sgn2);
-	                	qresitem(&Qpol2,AA2);
-	                	sgn = (sgn == sgn2 ? 0 : -1);
-	                	combinebp(N,len,cofac,QQ,AA,QQ2,AA2,hilf);
-	                	haveres = 1;
-	            	}
-            	}
+                else if(useBigprim && (cofac < bigpbound) && (cofac > 1)) {
+                    res = hashbigp(qsbigp,cofac,&Qpol,&Qpol2,hilf);
+                    if(res > 0) {
+                        if((++count2 & 0x3) == 1)
+                            if(doreport)
+                                tick(':');
+                        qresitem(&Qpol,AA);
+                        quadvalue(&Qpol2,QQ2,&sgn2);
+                        qresitem(&Qpol2,AA2);
+                        sgn = (sgn == sgn2 ? 0 : -1);
+                        combinebp(N,len,cofac,QQ,AA,QQ2,AA2,hilf);
+                        haveres = 1;
+                    }
+                }
 #endif /* BIGPRIMES */
-            	if(haveres) {
-	            	count++;
-	            	bm_insert(bmp,QQ,sgn,AA,hilf);
-	            	if(gausselim(bmp)) {
-	                	shrieks++;
-	                	if(doreport)
-	                		tick('!');
-	                	n = getfactor(N,len,bmp,fact,hilf);
-	                	if(n > 0) {
+                if(haveres) {
+                    count++;
+                    bm_insert(bmp,QQ,sgn,AA,hilf);
+                    if(gausselim(bmp)) {
+                        shrieks++;
+                        if(doreport)
+                            tick('!');
+                        n = getfactor(N,len,bmp,fact,hilf);
+                        if(n > 0) {
 #ifdef BIGPRIMES
-	                		if(useBigprim) {
-	                    		if(doreport)
-	                        		qs2mess(v,count,count2);
-	                		}
-	                		else
+                            if(useBigprim) {
+                                if(doreport)
+                                    qs2mess(v,count,count2);
+                            }
+                            else
 #endif
-		                    	if(doreport)
-		                    		qs1mess(v,count);
-	                		return(n);
-	                	}
-						if(shrieks >= maxshrieks)
-							break;
-	            	}
-            	}
-        	}
-    	}
-    	if(doreport)
-        	counttick(v,bmp);
-	}
-	/* check if N is a perfect square */
-	NN = hilf + 1;
-	hilf1 = hilf + len + 2;
-	cpyarr(N,len,NN);
-	k = is_square(NN,len,fact,hilf1);
+                                if(doreport)
+                                    qs1mess(v,count);
+                            return(n);
+                        }
+                        if(shrieks >= maxshrieks)
+                            break;
+                    }
+                }
+            }
+        }
+        if(doreport)
+            counttick(v,bmp);
+    }
+    /* check if N is a perfect square */
+    NN = hilf + 1;
+    hilf1 = hilf + len + 2;
+    cpyarr(N,len,NN);
+    k = is_square(NN,len,fact,hilf1);
     return(k);
 }
 /*---------------------------------------------------------------*/
@@ -1930,7 +1940,7 @@ SIEVEDATA *qsp;
             /* sieving with first root */
             xi0 = (r1 >= s ? r1 - s : r1 + (p-s));
             for(sptr=sieve+xi0,xi=xi0; xi<srange2; sptr+=p,xi+=p)
-            	*sptr += z;
+                *sptr += z;
             /* now calculate the other root of qpol */
             u = (p - r) + (p - b1);
             u *= ainv;
@@ -2146,8 +2156,8 @@ int len, q0len;
 
     len0 = (len > qqlen ? len : qqlen+1);
     NN = hilf; QQ = hilf + len0; aux = QQ + len0;
-    cpyarr(N,len,NN);            
-    cpyarr(qq,qqlen,QQ); 
+    cpyarr(N,len,NN);
+    cpyarr(qq,qqlen,QQ);
     if(jacobi(0,NN,len,QQ,qqlen,aux) == 1) {
         if(qqlen <= 2) {
             if(prime32(big2long(qq,qqlen)))
@@ -2306,26 +2316,25 @@ PRIVATE int ppexpo(B1,B2,xx)
 unsigned B1, B2;
 word2 *xx;
 {
-	unsigned m1,m2,k;
-	int len;
+    unsigned m1,m2,k;
+    int len;
 
-	m1 = intsqrt(B1) + 1;
+    m1 = intsqrt(B1) + 1;
     if(m1 < 2)
         m1 = 2;
     m2 = intsqrt(B2);
 
-	xx[0] = 1;
-	len = 1;
-	for(k=m1; k<=m2; k++)
-		len = multarr(xx,len,k,xx);
-	if(++B1 <= 2)
-        B1 = 3;
-	B1 |= 0x1;
-	for(k=B1; k<=B2; k+=2) {
-		if(prime16(k))
-			len = multarr(xx,len,k,xx);
-	}
-	return len;
+    xx[0] = 1;
+    len = 1;
+    for(k=m1; k<=m2; k++)
+        len = multarr(xx,len,k,xx);
+    if(B1 & 1)
+        B1++;
+    for(k=B1+1; k<=B2; k+=2) {
+        if(prime16(k))
+            len = multarr(xx,len,k,xx);
+    }
+    return len;
 }
 /*---------------------------------------------------------------*/
 PRIVATE int multlarr(x,n,a,y)
@@ -2410,7 +2419,7 @@ word2 *AA;
     }
     len = multbig(ww2,len,qinv,qlen,ww0,hilf);
     len = modbig(ww0,len,N,nlen,hilf);
-    cpyarr(ww0,len,AA); 
+    cpyarr(ww0,len,AA);
     return(AA[-1] = len);
 }
 /*---------------------------------------------------------------*/
@@ -2457,35 +2466,36 @@ PRIVATE int is_square(N,len,root,hilf)
 word2 *N, *root, *hilf;
 int len;
 {
-	unsigned M = 15015; /* M=3*5*7*11*13 */
-	unsigned a;
-	int k, rlen;
+    unsigned M = 15015; /* M=3*5*7*11*13 */
+    unsigned a;
+    int k, rlen;
 
-	if(len <= 0)
-		return 0;
-	if((N[0] & 1) == 0) {
-		if((N[0] & 0x3) != 0)
-			return 0;
-	}
-	else if((N[0] & 0x7) != 1)
-		return 0;
-	a = modarr(N,len,M);
-	if(jac(a,13) == -1 || jac(a,11) == -1 || jac(a,7) == -1 ||
-	   jac(a,5) == -1 || jac(a,3) == -1)
-		return 0;
-	k = bigsqrt(N,len,root,&rlen,hilf);
-	if(rlen == 0)
-		return k;
-	else
-		return 0;
+    if(len <= 0)
+        return 0;
+    if((N[0] & 1) == 0) {
+        if((N[0] & 0x3) != 0)
+            return 0;
+    }
+    else if((N[0] & 0x7) != 1)
+        return 0;
+    a = modarr(N,len,M);
+    if(jac(a,13) == -1 || jac(a,11) == -1 || jac(a,7) == -1 ||
+       jac(a,5) == -1 || jac(a,3) == -1)
+        return 0;
+    k = bigsqrt(N,len,root,&rlen,hilf);
+    if(rlen == 0)
+        return k;
+    else
+        return 0;
 }
 /*------------------------------------------------------------------*/
 PRIVATE truc Fecfactor(argn)
 int argn;
 {
-	struct vector *vec;
-	truc *ptr;
+    struct vector *vec;
+    truc *ptr;
     truc *argptr;
+    truc obj;
     word2 *N, *z, *xx, *x0, *cc, *hilf;
     word2 aa[2];
     unsigned u, v, bound, bigbound, bound2, anz;
@@ -2495,6 +2505,9 @@ int argn;
     ECN ecN;
     int  mhdiff, ret, usebpv;
 
+    /*
+    ** parse and examine arguments of ec_factorize
+    */
     argptr = argStkPtr-argn+1;
     if(argn >= 2 && *argStkPtr == zero) {
         doreport = 0;
@@ -2526,13 +2539,13 @@ int argn;
     bound = 1;
     bigbound = 1;
     if(argn>=2 && *FLAGPTR(argptr+1) == fVECTOR) {
-    	vec = (struct vector *)TAddress(argptr+1);
-	    n = vec->len;
+        vec = (struct vector *)TAddress(argptr+1);
+        n = vec->len;
         if(n != 2) {
             error(ecfactsym,"vector of length 2 expected",argptr[1]);
             return brkerr();
         }
-    	ptr = &(vec->ele0);
+        ptr = &(vec->ele0);
         if(chkints(ecfactsym,ptr,2) == aERROR)
             return(brkerr());
         n = bigref(ptr,&z,&sign);
@@ -2597,10 +2610,13 @@ int argn;
                 m = 10;
             bigbound = m*bound;
         }
-        else if(bigbound > 0x1000000) {
+        if(bigbound > 0x1000000) {
             bigbound = 0x1000000;
         }
     }
+    /*
+    ** memory allocation
+    */
     x0 = AriScratch;
     xx = x0 + 2*nlen + 2;
     cc = x0 + 4*nlen + 4;
@@ -2619,18 +2635,25 @@ int argn;
         else
             ec0mess(bound);
     }
+    /*
+    ** choose random curve
+    */
     u = 3 + random4(0x1000000);
     alen = long2big(u,aa);
     v = 1 + random2(64000);
+    /*
+    ** choose random initial point (x0,x0len)
+    */
     /* here nlen > 1 */
     x0[nlen-1] = 1;
     for(k=0; k<=nlen-2; k++)
         x0[k] = random2(0xFFFF);
     x0len = nlen;
+    obj = zero;
     for(k=0; k<anz; k++) {
         if(INTERRUPT) {
-			setinterrupt(0);
-        	break;
+            setinterrupt(0);
+            break;
         }
         if(doreport) {
             if(k % 100 == 99) {
@@ -2650,7 +2673,8 @@ int argn;
                 u = big2long(aa,alen);
                 ec2mess(u,bound);
             }
-            return mkint(0,AriBuf,n);
+            obj = mkint(0,AriBuf,n);
+            goto ausgang;
         }
         /* goto big prime variation */
         if(usebpv == 0)
@@ -2666,7 +2690,8 @@ int argn;
         else if(ret < 0) {
             cpyarr(ecN.cc,ecN.clen,AriBuf);
             n = ecN.clen;
-            return mkint(0,AriBuf,n);
+            obj = mkint(0,AriBuf,n);
+            goto ausgang;
         }
         bound2 = bigbound;
         ret = ecfactbpv(&ecN,Epoint,&bound2,mhdiff,xx,xlen,hilf);
@@ -2677,13 +2702,15 @@ int argn;
                 u = big2long(aa,alen);
                 ec3mess(u,bound2);
             }
-            return mkint(0,AriBuf,n);
+            obj = mkint(0,AriBuf,n);
+            goto ausgang;
         }
   skipbpv:
         x0len = incarr(x0,x0len,v);
         alen = incarr(aa,alen,1);
     }
-    return zero;
+  ausgang:
+    return obj;
 }
 /*------------------------------------------------------------------*/
 /*
@@ -2789,22 +2816,18 @@ word2 *xx, *hilf;
 int xlen, hdiff;
 unsigned *pbound;
 {
-    EPOINT Z;
+    EPOINT *pZ;
     EPOINT *pEtemp;
     word2 *zz;
     int k, ret, nlen;
     unsigned q, bound;
     int found = 0;
 
-    nlen = pecN->nlen;
-    Z.xx = hilf;
-    Z.yy = hilf + nlen + 1;
-    hilf += 2*nlen + 2;
-
-    cpyarr(xx,xlen,Z.xx);
-    Z.xlen = xlen;
-    Z.yy[0] = 1;
-    Z.ylen = 1;
+    pZ = pEpoint;
+    cpyarr(xx,xlen,pZ->xx);
+    pZ->xlen = xlen;
+    pZ->yy[0] = 1;
+    pZ->ylen = 1;
 
     pEtemp = pEpoint + 1;
     cpyarr(xx,xlen,pEtemp->xx);
@@ -2855,10 +2878,10 @@ unsigned *pbound;
             q += 2;
             k++;
         }
-        ret = ECNadd(pecN,&Z,pEpoint+k,hilf);
+        ret = ECNadd(pecN,pZ,pEpoint+k,hilf);
         if(ret == -2) {
-            xlen = Z.ylen;
-            zz = Z.yy;
+            xlen = pZ->ylen;
+            zz = pZ->yy;
             *pbound = q;
             found = 1;
             break;
@@ -2869,12 +2892,13 @@ unsigned *pbound;
         cpyarr(zz,xlen,xx);
         return(-xlen-1);
     }
-    else
+    else {
         return xlen;
+    }
 }
 /*-----------------------------------------------------------------*/
 /*
-** Given x, a an N, calculates c such that
+** Given x, a and N, calculates c such that
 **  c = x**3 + a*x**2 + x mod N
 ** a and N are handed in through pecN, the result
 ** c is stored in pecN
@@ -2936,7 +2960,7 @@ int xlen;
 **   >= 0, if result is an affine point
 **   -1, if Origin
 **   -2, if divisor of N detected
-**       This divisor is then stored in (pZ1->yy,pZ1->ylen)
+**   This divisor is then stored in (pZ1->yy,pZ1->ylen)
 */
 /*
     (x,y) := (x1,y1) add (x2,y2)
@@ -3064,6 +3088,10 @@ word2 *hilf;
 /*
 ** Calculates destructively Z := Z + Z
 ** Returns pZ->xlen
+**   >= 0, if result is an affine point
+**   -1, if Origin
+**   -2, if divisor of N detected
+**   This divisor is then stored in (pZ->yy,pZ->ylen)
 */
 /*
     z := 2*c*y1;
@@ -3161,6 +3189,14 @@ word2 *hilf;
     return pZ->xlen;
 }
 /*------------------------------------------------------------------*/
+/*
+** Calculates destructively Z := (ex,exlen)*Z
+** Returns pZ->xlen
+**   >= 0, if result is an affine point
+**   -1, if Origin
+**   -2, if divisor of N detected
+**   This divisor is then stored in (pZ->yy,pZ->ylen)
+*/
 PRIVATE int ECNmult(pecN, pZ, ex, exlen, hilf)
 ECN *pecN;
 EPOINT *pZ;
