@@ -2,7 +2,7 @@
 /* file aritz.c
 
 ARIBAS interpreter for Arithmetic
-Copyright (C) 1996-2004 O.Forster
+Copyright (C) 1996-2007 O.Forster
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -20,12 +20,12 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
 Address of the author
 
-	Otto Forster
-	Math. Institut der LMU
-	Theresienstr. 39
-	D-80333 Muenchen, Germany
+    Otto Forster
+    Math. Institut der LMU
+    Theresienstr. 39
+    D-80333 Muenchen, Germany
 
-Email	forster@mathematik.uni-muenchen.de
+Email   forster@mathematik.uni-muenchen.de
 WWW     http://www.mathematik.uni-muenchen.de
 */
 /****************************************************************/
@@ -38,6 +38,10 @@ WWW     http://www.mathematik.uni-muenchen.de
 **
 ** 2002-04-07:  created
 ** 2003-03-05:  gf2n functions
+** 2007-04-04:  gf2X functions
+** 2007-09-08:  bugfix in gf2X_mod
+** 2007-09-18:  gf2polsquare
+** 2007-10-01:  gf2X_divide
 */
 #include "common.h"
 
@@ -59,17 +63,17 @@ typedef struct {
 
 /*-----------------------------------------------------------------*/
 /* setbit and testbit suppose that vv is an array of word2 */
-#define setbit(vv,i)	vv[(i)>>4] |= (1 << ((i)&0xF))
-#define testbit(vv,i)	(vv[(i)>>4] & (1 << ((i)&0xF)))
+#define setbit(vv,i)    vv[(i)>>4] |= (1 << ((i)&0xF))
+#define testbit(vv,i)   (vv[(i)>>4] & (1 << ((i)&0xF)))
 /*-----------------------------------------------------------------*/
 #define MULTFLAG    0
-#define DIVFLAG		1
-#define MODFLAG		2
+#define DIVFLAG     1
+#define MODFLAG     2
 #define MODNFLAG    4
-#define DDIVFLAG	(DIVFLAG | MODFLAG)
+#define DDIVFLAG    (DIVFLAG | MODFLAG)
 /*-----------------------------------------------------------------*/
 
-PUBLIC void iniaritz	_((void));
+PUBLIC void iniaritz    _((void));
 
 PUBLIC truc gf2nzero, gf2none;
 PUBLIC truc gf2nintsym, gf2n_sym;
@@ -98,19 +102,24 @@ PRIVATE int chkpoldivargs   _((truc sym, truc *argptr));
 PRIVATE truc multintpols    _((truc *argptr, int mode));
 PRIVATE truc modintpols     _((truc *argptr, int mode));
 
-PRIVATE int gf2polmod	_((word2 *x, int n, word2 *y, int m));
-PRIVATE int gf2polmult	_((word2 *x, int n, word2 *y, int m, word2 *z));
+PRIVATE int gf2polmod   _((word2 *x, int n, word2 *y, int m));
+PRIVATE int gf2polmult  _((word2 *x, int n, word2 *y, int m, word2 *z));
+PRIVATE int gf2poldivide _((word2 *x, int n, word2 *y, int m, word2 *z, int *rlenptr));
+PRIVATE int gf2poldiv   _((word2 *x, int n, word2 *y, int m, word2 *z));
 PRIVATE int gf2ninverse _((word2 *x, int n, word2 *z, word2 *uu));
 PRIVATE int gf2npower   _((word2 *x, int n, word2 *y, int m,
                     word2 *z, word2 *hilf));
 PRIVATE int gf2polsquare    _((word2 *x, int n, word2 *z));
-PRIVATE int gf2polgcd	_((word2 *x, int n, word2 *y, int m));
+PRIVATE int gf2polmodpow    _((word2 *x, int n, word2 *y, int m,
+                    word2 *f, int flen, word2 *z, word2 *hilf));
+PRIVATE int gf2polgcd   _((word2 *x, int n, word2 *y, int m));
 PRIVATE int gf2polgcdx  _((word2 *x, int n, word2 *y, int m,
                     word2 *z, int *zlenptr, word2 *hilf));
 PRIVATE int gf2polirred _((word2 *x, int n, word2 *y, word2 *hilf));
+PRIVATE int gf2polirred1 _((word2 *x, int n, word2 *y, word2 *hilf));
 PRIVATE int gf2ntrace   _((word2 *x, int n));
 PRIVATE int shiftleft1  _((word2 *x, int n));
-PRIVATE int bitxorshift	_((word2 *x, int n, word2 *y, int m, int s));
+PRIVATE int bitxorshift _((word2 *x, int n, word2 *y, int m, int s));
 PRIVATE unsigned gf2polfindirr  _((int n));
 
 PRIVATE truc gf2ndegsym, gf2npolsym, gf2ninisym, gf2ntrsym, maxgf2nsym;
@@ -121,6 +130,17 @@ PRIVATE truc Fgf2ntrace _((void));
 PRIVATE truc Fmaxgf2n   _((void));
 PRIVATE int gf2nmod _((word2 *x, int n));
 
+PRIVATE truc gf2Xmulsym, gf2Xsqsym, gf2Xddivsym, gf2Xdivsym, gf2Xmodsym, gf2Xgcdsym;
+PRIVATE truc gf2Xmpowsym, gf2Xprimsym;
+PRIVATE truc Fgf2Xmult   _((void));
+PRIVATE truc Fgf2Xsquare _((void));
+PRIVATE truc Fgf2Xdivide _((void));
+PRIVATE truc Fgf2Xdiv    _((void));
+PRIVATE truc Fgf2Xmod    _((void));
+PRIVATE truc Fgf2Xgcd    _((void));
+PRIVATE truc Fgf2Xmodpow    _((void));
+PRIVATE truc Fgf2Xprimtest  _((void));
+
 /*----------------------------------------------------*/
 PRIVATE void fp2Dmult _((FP2D *pfp2D, PAIRXY *pZ1, PAIRXY *pZ2, word2 *hilf));
 PRIVATE void fp2Dsquare _((FP2D *pfp2D, PAIRXY *pZ, word2 *hilf));
@@ -135,7 +155,7 @@ PRIVATE int fpSqrt58    _((word2 *pp, int plen, word2 *aa, int alen,
 PRIVATE int fpSqrt14    _((word2 *pp, int plen, word2 *aa, int alen,
                     word2 *zz, word2 *hilf));
 PRIVATE unsigned fp_sqrt14  _((unsigned p, unsigned a));
-PRIVATE void fp2pow 	_((unsigned p, unsigned D, unsigned *uu, unsigned n));
+PRIVATE void fp2pow     _((unsigned p, unsigned D, unsigned *uu, unsigned n));
 
 PRIVATE truc Ffpsqrt  _((void));
 
@@ -144,16 +164,15 @@ PRIVATE truc fpsqrtsym;
 /* #define NAUSKOMM */
 #ifdef NAUSKOMM
 PRIVATE truc gggsym, gg1sym, gg2sym;
-PRIVATE truc Fggg	_((void));
-PRIVATE truc Fgg1	_((void));
-PRIVATE truc Fgg2	_((void));
+PRIVATE truc Fggg   _((void));
+PRIVATE truc Fgg1   _((void));
+PRIVATE truc Fgg2   _((void));
 #endif
 
-#define GF2NINTEGER
+#define GF2XARITH
 /*------------------------------------------------------------------*/
 PUBLIC void iniaritz()
 {
-#ifdef GF2NINTEGER
     word2 x[1];
 
     gf2nzero   = mk0gf2n(x,0);
@@ -167,7 +186,17 @@ PUBLIC void iniaritz()
     gf2ndegsym = newsymsig("gf2n_degree", sFBINARY, (wtruc)Fgf2ndegree, s_0);
     gf2npolsym = newsymsig("gf2n_fieldpol",sFBINARY,(wtruc)Fgf2nfieldpol,s_0);
     maxgf2nsym = newsymsig("max_gf2nsize",sFBINARY, (wtruc)Fmaxgf2n, s_0);
-#endif GF2NINTEGER
+
+#ifdef GF2XARITH
+    gf2Xmulsym = newsymsig("gf2X_mult", sFBINARY, (wtruc)Fgf2Xmult, s_2);
+    gf2Xsqsym  = newsymsig("gf2X_square", sFBINARY, (wtruc)Fgf2Xsquare, s_1);
+    gf2Xddivsym= newsymsig("gf2X_divide", sFBINARY, (wtruc)Fgf2Xdivide, s_2);
+    gf2Xdivsym = newsymsig("gf2X_div", sFBINARY, (wtruc)Fgf2Xdiv, s_2);
+    gf2Xmodsym = newsymsig("gf2X_mod", sFBINARY, (wtruc)Fgf2Xmod, s_2);
+    gf2Xgcdsym = newsymsig("gf2X_gcd", sFBINARY, (wtruc)Fgf2Xgcd, s_2);
+    gf2Xmpowsym = newsymsig("gf2X_modpower", sFBINARY, (wtruc)Fgf2Xmodpow, s_3);
+    gf2Xprimsym = newsymsig("gf2X_primetest",sFBINARY,(wtruc)Fgf2Xprimtest, s_1);
+#endif /* GF2XARITH */
 
 #ifdef POLYARITH
     polmultsym = newsymsig("pol_mult",  sFBINARY, (wtruc)Fpolmult, s_2);
@@ -181,9 +210,9 @@ PUBLIC void iniaritz()
     fpsqrtsym = newsymsig("gfp_sqrt", sFBINARY, (wtruc)Ffpsqrt, s_2);
 
 #ifdef NAUSKOMM
-	gggsym = newsymsig("ggg", sFBINARY, (wtruc)Fggg, s_2);
-	gg1sym = newsymsig("gg1", sFBINARY, (wtruc)Fgg1, s_2);
-	gg2sym = newsymsig("gg2", sFBINARY, (wtruc)Fgg2, s_3);
+    gggsym = newsymsig("ggg", sFBINARY, (wtruc)Fggg, s_2);
+    gg1sym = newsymsig("gg1", sFBINARY, (wtruc)Fgg1, s_2);
+    gg2sym = newsymsig("gg2", sFBINARY, (wtruc)Fgg2, s_3);
 #endif
 }
 /*-------------------------------------------------------------------*/
@@ -192,8 +221,8 @@ PRIVATE truc Ffpsqrt()
     word2 *pp, *aa, *hilf;
     int plen, alen, len, sign;
 
-	if(chkints(fpsqrtsym,argStkPtr-1,2) == aERROR)
-		return(brkerr());
+    if(chkints(fpsqrtsym,argStkPtr-1,2) == aERROR)
+        return(brkerr());
     plen = bigref(argStkPtr-1,&pp,&sign);
     if(plen == 0 || (pp[0] & 1) == 0 || (plen == 1 && pp[0] <= 2)) {
         error(fpsqrtsym,err_oddprim,argStkPtr[-1]);
@@ -229,8 +258,8 @@ PRIVATE truc Fggg()
     size_t N;
     int plen, alen, len, sign;
 
-	if(chkints(gggsym,argStkPtr-1,2) == aERROR)
-		return(brkerr());
+    if(chkints(gggsym,argStkPtr-1,2) == aERROR)
+        return(brkerr());
     aa = AriScratch;
     plen = bigref(argStkPtr-1,&pp,&sign);
     alen = bigretr(argStkPtr,aa,&sign);
@@ -258,8 +287,8 @@ PRIVATE truc Fgg1()
     size_t N;
     int plen, alen, len, sign;
 
-	if(chkints(gg1sym,argStkPtr-1,2) == aERROR)
-		return(brkerr());
+    if(chkints(gg1sym,argStkPtr-1,2) == aERROR)
+        return(brkerr());
     aa = AriScratch;
     plen = bigref(argStkPtr-1,&pp,&sign);
     alen = bigretr(argStkPtr,aa,&sign);
@@ -291,8 +320,8 @@ PRIVATE truc Fgg2()
     FP2D fp2D;
     PAIRXY Z;
 
-	if(chkints(gg1sym,argStkPtr-2,3) == aERROR)
-		return(brkerr());
+    if(chkints(gg1sym,argStkPtr-2,3) == aERROR)
+        return(brkerr());
     D = hilf = AriScratch;
     plen = bigref(argStkPtr-2,&pp,&sign);
     dlen = bigretr(argStkPtr-1,D,&sign);
@@ -820,7 +849,7 @@ int exlen;
     allowintr = (plen >= 16 && (exlen + plen >= 256) ? 1 : 0);
     for(k=bitl-2; k>=0; k--) {
         fp2Dsquare(pfp2D,pZ,hilf);
-    	if(testbit(ex,k))
+        if(testbit(ex,k))
             fp2Dmult(pfp2D,pZ,&Z0,hilf);
         if(allowintr && INTERRUPT) {
             setinterrupt(0);
@@ -867,9 +896,9 @@ int mode;
     truc *workptr, *w2ptr, *ptr, *wptr;
     struct vector *vecptr;
     truc obj;
-	word2 *x, *y, *zz, *aa, *hilf;
+    word2 *x, *y, *zz, *aa, *hilf;
     int len, len1, len2, k, i, j0, j1;
-	int n1, n2, n3, n, m, sign, sign1, sign2, sign3;
+    int n1, n2, n3, n, m, sign, sign1, sign2, sign3;
     unsigned mlen, offshilf;
 
     len = *VECLENPTR(argptr);
@@ -930,13 +959,13 @@ int mode;
             n1 = bigref(workptr+i,&x,&sign1);
             n2 = bigref(w2ptr+(k-i),&y,&sign2);
             m = multbig(x,n1,y,n2,AriScratch,hilf);
-           	sign1 = (sign1 == sign2 ? 0 : MINUSBYTE);
+            sign1 = (sign1 == sign2 ? 0 : MINUSBYTE);
             n = addsarr(aa,n,sign,AriScratch,m,sign1,&sign);
         }
         if(mode & MODNFLAG) {
             n = modbig(aa,n,zz,n3,hilf);
             if(n && (sign != sign3)) {
-		        n = sub1arr(aa,n,zz,n3);
+                n = sub1arr(aa,n,zz,n3);
                 sign = sign3;
             }
         }
@@ -953,8 +982,8 @@ int mode;
     return obj;
   ovflexit:
     workStkPtr = workptr-1;
-	error(polmultsym,err_ovfl,voidsym);
-	return(brkerr());
+    error(polmultsym,err_ovfl,voidsym);
+    return(brkerr());
 }
 /*------------------------------------------------------------------*/
 PRIVATE truc Fpolmod()
@@ -1060,7 +1089,7 @@ int mode;
             m2 = bigref(argStkPtr,&zz,&sign2);
             n = modbig(aa,n,zz,m2,AriScratch);
             if(n && (sign != sign2)) {
-		        n = sub1arr(aa,n,zz,m2);
+                n = sub1arr(aa,n,zz,m2);
                 sign = sign2;
             }
         }
@@ -1132,8 +1161,8 @@ int mode;
     return obj;
   ovflexit:
     workStkPtr = workptr-1;
-	error(polmodsym,err_ovfl,voidsym);
-	return(brkerr());
+    error(polmodsym,err_ovfl,voidsym);
+    return(brkerr());
 }
 /*------------------------------------------------------------------*/
 PRIVATE int chkpolmultargs(sym,argptr)
@@ -1242,12 +1271,12 @@ static word2 spreadbyte[256] = {
 PUBLIC truc addgf2ns(ptr)
 truc *ptr;
 {
-	word2 *y;
-	int n, m, deg;
-	int sign;
+    word2 *y;
+    int n, m, deg;
+    int sign;
 
-	n = bigretr(ptr,AriBuf,&sign);
-	m = bigref(ptr+1,&y,&sign);
+    n = bigretr(ptr,AriBuf,&sign);
+    m = bigref(ptr+1,&y,&sign);
     deg = gf2nField.deg;
     if(deg < bit_length(AriBuf,n) || deg < bit_length(y,m)) {
         error(plussym,"gf2nint summand too big",voidsym);
@@ -1360,11 +1389,11 @@ truc *ptr;
 */
 PRIVATE truc Fgf2nint()
 {
-	word2 *x;
-	byte *bpt;
-	unsigned u;
-	unsigned len;
-	int i, n, flg, sign;
+    word2 *x;
+    byte *bpt;
+    unsigned u;
+    unsigned len;
+    int i, n, flg, sign;
 
     flg = *FLAGPTR(argStkPtr);
 
@@ -1372,25 +1401,25 @@ PRIVATE truc Fgf2nint()
         n = bigretr(argStkPtr,AriBuf,&sign);
     }
     else if(flg == fBYTESTRING) {
-    	len = *STRLENPTR(argStkPtr);
-	    if(len >= aribufSize*2 - 2) {
-		    error(gf2n_sym,err_2long,mkfixnum(len));
-    		return(brkerr());
-	    }
-    	bpt = (byte *)STRINGPTR(argStkPtr);
-	    n = len / 2;
-    	x = AriBuf;
-	    for(i=0; i<n; i++) {
-		    u = bpt[1];
-    		u <<= 8;
-	    	u += bpt[0];
-		    *x++ = u;
-    		bpt += 2;
-	    }
-    	if(len & 1) {
-	    	*x = *bpt;
-		    n++;
-    	}
+        len = *STRLENPTR(argStkPtr);
+        if(len >= aribufSize*2 - 2) {
+            error(gf2n_sym,err_2long,mkfixnum(len));
+            return(brkerr());
+        }
+        bpt = (byte *)STRINGPTR(argStkPtr);
+        n = len / 2;
+        x = AriBuf;
+        for(i=0; i<n; i++) {
+            u = bpt[1];
+            u <<= 8;
+            u += bpt[0];
+            *x++ = u;
+            bpt += 2;
+        }
+        if(len & 1) {
+            *x = *bpt;
+            n++;
+        }
     }
     else {
         error(gf2n_sym,err_int,voidsym);
@@ -1427,7 +1456,7 @@ PRIVATE truc Fgf2ninit()
     gf2nField.deg = n;
     gf2nField.ftail = u;
 
-    m = (n+1)/16 + 1;
+    m = n/16 + 1;
     x = AriBuf;
     setarr(x,m,0);
     x[0] = u;
@@ -1500,7 +1529,7 @@ PRIVATE truc Fmaxgf2n()
 /*-------------------------------------------------------------------*/
 PRIVATE truc Fgf2ntrace()
 {
-	int n, t, sign;
+    int n, t, sign;
     word2 *x;
 
     if(*FLAGPTR(argStkPtr) != fGF2NINT) {
@@ -1509,8 +1538,8 @@ PRIVATE truc Fgf2ntrace()
     }
     x = AriBuf;
     n = bigretr(argStkPtr,x,&sign);
-	if(n == aERROR)
-		return brkerr();
+    if(n == aERROR)
+        return brkerr();
     if(gf2nField.deg < bit_length(x,n)) {
         error(gf2ntrsym,"gf2nint argument too big",voidsym);
         return brkerr();
@@ -1519,6 +1548,218 @@ PRIVATE truc Fgf2ntrace()
     return mkfixnum(t);
 }
 /*------------------------------------------------------------------*/
+PRIVATE truc Fgf2Xmult()
+{
+    int n,m,len,sign;
+    word2 *x, *y;
+
+    n = bigref(argStkPtr-1,&x,&sign);
+    if (n == aERROR)
+        goto errexit;
+    m = bigref(argStkPtr,&y,&sign);
+    if (m == aERROR)
+        goto errexit;
+    if(n + m >= aribufSize) {
+        error(gf2Xmulsym,err_ovfl,voidsym);
+        return(brkerr());
+    }
+    else if(!n || !m)
+        return(zero);
+    len = gf2polmult(x,n,y,m,AriBuf);
+    return mkint(0,AriBuf,len);
+
+  errexit:
+    error(gf2Xmulsym,err_intt,voidsym);
+    return brkerr();
+}
+/*------------------------------------------------------------------*/
+PRIVATE truc Fgf2Xsquare()
+{
+    int n,len,sign;
+    word2 *x;
+
+    n = bigref(argStkPtr,&x,&sign);
+    if (n == aERROR) {
+        error(gf2Xsqsym,err_intt,voidsym);
+        return brkerr();
+    }
+    if (2*n >= aribufSize) {
+        error(gf2Xmulsym,err_ovfl,voidsym);
+        return(brkerr());
+    }
+    else if (!n)
+        return zero;
+
+    len = gf2polsquare(x,n,AriBuf);
+    return mkint(0,AriBuf,len);
+}
+/*------------------------------------------------------------------*/
+PRIVATE truc Fgf2Xdivide()
+{
+    int n,m,len,sign,rlen;
+    word2 *x, *y;
+    truc obj, vec;
+    truc *vptr;
+
+    x = AriScratch;
+    n = bigretr(argStkPtr-1,x,&sign);
+    if (n == aERROR)
+        goto errexit;
+    else if(n >= aribufSize) {
+        error(gf2Xddivsym,err_ovfl,voidsym);
+        return(brkerr());
+    }
+    m = bigref(argStkPtr,&y,&sign);
+    if (m == aERROR)
+        goto errexit;
+    else if(m == 0) {
+        error(gf2Xddivsym,err_div,voidsym);
+        return brkerr();
+    }
+    len = gf2poldivide(x,n,y,m,AriBuf,&rlen);
+    y = AriBuf + len;
+    cpyarr(x,rlen,y);
+
+    obj = mkint(0,AriBuf,len);
+    WORKpush(obj);
+    obj = mkint(0,y,rlen);
+    WORKpush(obj);
+    vec = mkvect0(2);
+    vptr = VECTOR(vec);
+    vptr[1] = WORKretr();
+    vptr[0] = WORKretr();
+    return vec;
+
+  errexit:
+    error(gf2Xddivsym,err_intt,voidsym);
+    return brkerr();
+}
+/*------------------------------------------------------------------*/
+PRIVATE truc Fgf2Xdiv()
+{
+    int n,m,len,sign;
+    word2 *x, *y;
+
+    x = AriScratch;
+    n = bigretr(argStkPtr-1,x,&sign);
+    if (n == aERROR)
+        goto errexit;
+    m = bigref(argStkPtr,&y,&sign);
+    if (m == aERROR)
+        goto errexit;
+    if (m > n)
+        return zero;
+    else if(n >= aribufSize) {
+        error(gf2Xdivsym,err_ovfl,voidsym);
+        return(brkerr());
+    }
+    else if(!m) {
+        error(gf2Xdivsym,err_div,voidsym);
+        return brkerr();
+    }
+    len = gf2poldiv(x,n,y,m,AriBuf);
+    return mkint(0,AriBuf,len);
+
+  errexit:
+    error(gf2Xdivsym,err_intt,voidsym);
+    return brkerr();
+}
+/*------------------------------------------------------------------*/
+PRIVATE truc Fgf2Xmod()
+{
+    int n,m,len,sign;
+    word2 *x, *y;
+
+    x = AriBuf;
+    n = bigretr(argStkPtr-1,x,&sign);
+    if (n == aERROR)
+        goto errexit;
+    m = bigref(argStkPtr,&y,&sign);
+    if (m == aERROR)
+        goto errexit;
+    if(n >= aribufSize) {
+        error(gf2Xmodsym,err_ovfl,voidsym);
+        return(brkerr());
+    }
+    else if(!m) {
+        error(gf2Xmodsym,err_div,voidsym);
+        return brkerr();
+    }
+    len = gf2polmod(x,n,y,m);
+    return mkint(0,AriBuf,len);
+
+  errexit:
+    error(gf2Xmodsym,err_intt,voidsym);
+    return brkerr();
+}
+/*------------------------------------------------------------------*/
+PRIVATE truc Fgf2Xgcd()
+{
+    int n,m,len,sign;
+    word2 *x, *y;
+
+    x = AriBuf;
+    y = AriScratch;
+    n = bigretr(argStkPtr-1,x,&sign);
+    if (n == aERROR)
+        goto errexit;
+    m = bigretr(argStkPtr,y,&sign);
+    if (m == aERROR)
+        goto errexit;
+    len = gf2polgcd(x,n,y,m);
+    return mkint(0,AriBuf,len);
+
+  errexit:
+    error(gf2Xgcdsym,err_intt,voidsym);
+    return brkerr();
+}
+/*------------------------------------------------------------------*/
+PRIVATE truc Fgf2Xmodpow()
+{
+    word2 *x, *y, *f;
+    int n,m,flen,len,sign;
+    int flg;
+    truc symb;
+
+    n = bigref(argStkPtr-2,&x,&sign);
+    if (n == aERROR) {
+        symb = argStkPtr[-2];
+        goto errexit;
+    }
+    flg = *FLAGPTR(argStkPtr-1);
+    if (flg != fFIXNUM && flg != fBIGNUM) {
+        error(gf2Xmpowsym,err_int,argStkPtr[-1]);
+        return brkerr();
+    }
+    flen = bigref(argStkPtr,&f,&sign);
+    if (flen == aERROR) {
+        symb = argStkPtr[0];
+        goto errexit;
+    }
+    else if (flen >= aribufSize/2 - 1) {
+        error(gf2Xmpowsym,err_ovfl,argStkPtr[0]);
+        return brkerr();
+    }
+    m = bigref(argStkPtr-1,&y,&sign);
+    if (m == aERROR) {
+        symb = argStkPtr[-1];
+        goto errexit;
+    }
+    else if (sign) {
+/******************************/
+/* TODO: negative exponent */
+        error(gf2Xmpowsym,err_pint,argStkPtr[-1]);
+        return brkerr();
+    }
+    len = gf2polmodpow(x,n,y,m,f,flen,AriBuf,AriScratch);
+
+    return mkint(0,AriBuf,len);
+
+  errexit:
+    error(gf2Xmpowsym,err_intt,symb);
+    return brkerr();
+}
+/*------------------------------------------------------------------*/
 /*
 ** P := (x,n) and Q := (y,m) are considered as polynomials over GF(2)
 ** Calculates P mod Q; works destructively on x; returns new length
@@ -1527,22 +1768,88 @@ PRIVATE int gf2polmod(x,n,y,m)
 word2 *x, *y;
 int n,m;
 {
-	int N, M, k, s;
+    int N, M, k, s;
 
     if(m == 0)
-		return n;
+        return n;
 
     N = bit_length(x,n) - 1;
     M = bit_length(y,m) - 1;
     if(M > N)
-		return n;
-	for(k=N; k>=M; --k) {
-		if(testbit(x,k)) {
-			s = k-M;
-			n = bitxorshift(x,n,y,m,s);
-		}
-	}
-	return n;
+        return n;
+    for(k=N; k>=M; --k) {
+        if(testbit(x,k)) {
+            s = k-M;
+            n = bitxorshift(x,n,y,m,s);
+        }
+    }
+    return n;
+}
+/*------------------------------------------------------------------*/
+/*
+** P := (x,n) and Q := (y,m) are considered as polynomials over GF(2)
+** Calculates P div Q; works destructively on x;
+** Result returned in z; return value=zlen
+*/
+PRIVATE int gf2poldiv(x,n,y,m,z)
+word2 *x, *y, *z;
+int n,m;
+{
+    int N, M, k, s, zlen;
+
+    if(m == 0)
+        return n;
+
+    N = bit_length(x,n) - 1;
+    M = bit_length(y,m) - 1;
+    if(M > N)
+        return 0;
+    zlen = (N - M)/16 + 1;
+    setarr(z,zlen,0);
+    for(k=N; k>=M; --k) {
+        if(testbit(x,k)) {
+            s = k-M;
+            n = bitxorshift(x,n,y,m,s);
+            setbit(z,s);
+        }
+    }
+    return zlen;
+}
+/*------------------------------------------------------------------*/
+/*
+** P := (x,n) and Q := (y,m) are considered as polynomials over GF(2)
+** Calculates P div Q; works destructively on x;
+** Quotient returned in z; return value=zlen
+** x becomes rest, ist len is returned in *rlentpr
+*/
+PRIVATE int gf2poldivide(x,n,y,m,z,rlenptr)
+word2 *x, *y, *z;
+int n,m;
+int *rlenptr;
+{
+    int N, M, k, s, zlen;
+
+    if(m == 0) {    /* division by zero */
+        *rlenptr = n;
+        return 0;
+    }
+    N = bit_length(x,n) - 1;
+    M = bit_length(y,m) - 1;
+    if(M > N) {
+        *rlenptr = n;
+        return 0;
+    }
+    zlen = (N - M)/16 + 1;
+    setarr(z,zlen,0);
+    for(k=N; k>=M; --k) {
+        if(testbit(x,k)) {
+            s = k-M;
+            n = bitxorshift(x,n,y,m,s);
+            setbit(z,s);
+        }
+    }
+    *rlenptr = n;
+    return zlen;
 }
 /*------------------------------------------------------------------*/
 PRIVATE int gf2ntrace(x,n)
@@ -1611,30 +1918,30 @@ int n,m,s;
     if(!m)
         return n;
 
-	s0 = (s >> 4);
+    s0 = (s >> 4);
     s1 = (s & 0xF);
 
-	if(s1 == 0) {
-		for(k=0, k1=s0-1; k<m; k++) {
-			x[++k1] ^= y[k];
-		}
+    if(s1 == 0) {
+        for(k=0, k1=s0-1; k<m; k++) {
+            x[++k1] ^= y[k];
+        }
     }
     else {
-		t1 = 16 - s1;
+        t1 = 16 - s1;
         x[s0] ^= (y[0] << s1);
-    	for(k=1, k1=s0; k<m; k++) {
+        for(k=1, k1=s0; k<m; k++) {
             x[++k1] ^= ((y[k] << s1) | (y[k-1] >> t1));
         }
         u = (y[m-1] >> t1);
         if(u) {
-			x[++k1] ^= u;
+            x[++k1] ^= u;
         }
-	}
+    }
     if(k1 >= n)
         n = k1+1;
     while((n > 0) && (x[n-1] == 0))
         n--;
-	return n;
+    return n;
 }
 /*------------------------------------------------------------------*/
 /*
@@ -1676,7 +1983,7 @@ int n;
 /*------------------------------------------------------------------*/
 /*
 ** Multiplies gf2pols (x,n) and (y,m);
-** does not touch (x,n) or (y,m)
+** does not alter (x,n) or (y,m)
 ** Result returned in z
 */
 PRIVATE int gf2polmult(x,n,y,m,z)
@@ -1695,29 +2002,34 @@ int n,m;
             n1 = bitxorshift(z,n1,x,n,k);
         }
     }
-	return n1;
+    return n1;
 }
 /*------------------------------------------------------------------*/
+/*
+** Squares the gf2pol (x,n)
+** Result returned in z
+*/
 PRIVATE int gf2polsquare(x,n,z)
 word2 *x, *z;
 int n;
 {
-    byte *bptr;
     int k, N;
     unsigned u;
 
     if(n == 0)
         return 0;
-    N = 2*n - 2;
-    bptr = (byte *)x;
-    for(k=0; k<N; k++) {
-        u = *bptr++;
-        z[k] = spreadbyte[u];
+    N = 0;
+    for(k=0; k<n-1; k++) {
+        u = x[k];
+        z[N] = spreadbyte[u & 0x00FF];
+        N++;
+        z[N] = spreadbyte[(u >> 8) & 0x00FF];
+        N++;
     }
     u = x[n-1];
     z[N] = spreadbyte[u & 0x00FF];
     N++;
-    u >>= 8;
+    u = (u >> 8) & 0x00FF;
     if(u) {
         z[N] = spreadbyte[u];
         N++;
@@ -1745,7 +2057,7 @@ int n;
     setbit(y,deg);
 
     uu = y + m + 1;
-	n = gf2polgcdx(x,n,y,m,z,&zlen,uu);
+    n = gf2polgcdx(x,n,y,m,z,&zlen,uu);
     if(x[0] != 1 || n != 1) {
         return 0;
     }
@@ -1785,6 +2097,39 @@ int n, m;
 }
 /*------------------------------------------------------------------*/
 /*
+** gf2pol (x,n) is raised to power (y,n) modulo (f,flen)
+** Result in z; hilf is an auxiliary array
+** (x,n), (y,m) and (f,flen) are not altered
+*/
+PRIVATE int gf2polmodpow(x,n,y,m,f,flen,z,hilf)
+word2 *x,*y,*z,*f,*hilf;
+int n, m, flen;
+{
+    int N, k, exlen;
+
+    if(m == 0) {
+        z[0] = 1;
+        return 1;
+    }
+    else if(n == 0)
+        return 0;
+    exlen = bit_length(y,m);
+    cpyarr(x,n,z);
+    N = n;
+    for(k=exlen-2; k>=0; k--) {
+        cpyarr(z,N,hilf);
+        N = gf2polsquare(hilf,N,z);
+        N = gf2polmod(z,N,f,flen);
+        if(testbit(y,k)) {
+            cpyarr(z,N,hilf);
+            N = gf2polmult(hilf,N,x,n,z);
+            N = gf2polmod(z,N,f,flen);
+        }
+    }
+    return N;
+}
+/*------------------------------------------------------------------*/
+/*
 ** Calculates greatest common divisor of gf2pols (x,n) and (y,m);
 ** Works destructively on x and y
 ** Result is returned in x
@@ -1802,7 +2147,7 @@ int n,m;
         }
         m = gf2polmod(y,m,x,n);
     }
-	return n;
+    return n;
 }
 /*-----------------------------------------------------------------*/
 /*
@@ -1872,6 +2217,80 @@ int *zlenptr;
     return n;
 }
 /*-----------------------------------------------------------------*/
+PRIVATE truc Fgf2Xprimtest()
+{
+    int n, sign, res;
+    word2 *x;
+
+    n = bigref(argStkPtr,&x,&sign);
+    if (n == aERROR) {
+        error(gf2Xprimsym,err_intt,argStkPtr[0]);
+        return brkerr();
+    }
+    res = gf2polirred1(x,n,AriBuf,AriScratch);
+    return (res ? true : false);
+}
+/*-----------------------------------------------------------------*/
+/*
+** Tests whether the gf2 polynomial given by (x,n) is irreducible.
+** yy and zz are auxiliary arrays needed for intermediate calculations
+*/
+PRIVATE int gf2polirred1(x,n,yy,zz)
+word2 *x, *yy, *zz;
+int n;
+{
+    int m, m1, N, N0, k, mode;
+    word2 xi;
+    word2 *x2k;
+
+    if (n == 0)
+        return 0;
+    if ((x[0] & 1) == 0)  /* polynomial divisible by X */
+        return 0;
+
+    N = bit_length(x,n) - 1;
+    N0 = 5*intsqrt(N);  /* somewhat arbitrary */
+    if (N0 < N/3) {
+        mode = 1;
+    }
+    else {
+        mode = 0;
+        N0 = N/2;
+    }
+    x2k = zz + n + 1;
+    xi = 2;
+    m = 1;
+    x2k[0] = xi;
+    for(k=1; k<=N0; k++) {
+        m = gf2polsquare(x2k,m,yy);
+        m = gf2polmod(yy,m,x,n);
+        if((m == 1 && yy[0] == xi) || m == 0)
+            return 0;
+        cpyarr(yy,m,x2k);
+        yy[0] ^= xi;
+        cpyarr(x,n,zz);
+        m1 = gf2polgcd(yy,m,zz,n);
+        if(m1 != 1 || yy[0] != 1) {
+            return 0;
+        }
+    }
+    if (mode == 0)
+        return 1;
+    for(k=N0+1; k<N; k++) {
+        m = gf2polsquare(x2k,m,yy);
+        m = gf2polmod(yy,m,x,n);
+        if((yy[0] == xi && m == 1) || m == 0)
+            return 0;
+        cpyarr(yy,m,x2k);
+    }
+    m = gf2polsquare(x2k,m,yy);
+    m = gf2polmod(yy,m,x,n);
+    if (yy[0] == xi && m == 1)
+        return 1;
+    else
+        return 0;
+}
+/*-----------------------------------------------------------------*/
 /*
 ** Tests whether the gf2 polynomial given by (x,n) is irreducible.
 ** yy and zz are auxiliary arrays needed for intermediate calculations
@@ -1901,10 +2320,15 @@ int n;
         if(m == 1 && x2k[0] == xi)
             return 0;
         cpyarr(x2k,m,yy);
-        x2k[0] ^= xi;
-        if(m == 0)
-            m = 1;
-        m1 = gf2polgcd(x2k,m,zz,n);
+        if (m == 0) {
+            x2k[0] = xi;
+            m1 = 1;
+        }
+        else {
+            x2k[0] ^= xi;
+            m1 = m;
+        }
+        m1 = gf2polgcd(x2k,m1,zz,n);
         if(m1 != 1 || x2k[0] != 1) {
             return 0;
         }
